@@ -1,40 +1,47 @@
 package Mojolicious::Plugin::MethodOverride;
 
+use 5.010;
 use Mojo::Base 'Mojolicious::Plugin';
 
-our $VERSION = '0.041';
+use Scalar::Util qw(weaken);
+
+our $VERSION = '0.050';
 
 sub register {
     my ($self, $app, $conf) = @_;
+
     my $header =
         exists $conf->{header} ? $conf->{header} : 'X-HTTP-Method-Override';
     my $param = exists $conf->{param} ? $conf->{param} : undef;
-    my $hook = $Mojolicious::VERSION < 3.84 ?   # support ancient Mojolicious
-        'after_static_dispatch' : 'before_routes';
 
     $app->hook(
-        $hook => sub {
-            my $req = $_[0]->req;
+        after_build_tx => sub {
+            my $tx = shift;
 
-            return unless $req->method eq 'POST';
+            weaken $tx;
 
-            my $method = defined $header && $req->headers->header($header);
+            $tx->req->content->on(body => sub {
+                my $req = $tx->req;
 
-            if ($method) {
-                $req->headers->remove($header);
-            }
-            elsif (defined $param) {
-                for ($req->url->query) {
-                    $method = $_->param($param)
-                        or return 1;
-                    $_->remove($param);
+                return unless $req->method eq 'POST';
+
+                my $method = defined $header && $req->headers->header($header);
+
+                if ($method) {
+                    $req->headers->remove($header);
                 }
-            }
+                elsif (defined $param) {
+                    my $q = $req->url->query;
 
-            if ($method and $method =~ /^[A-Za-z]+$/) {
-                $app->log->debug(($header // $param) . ': ' . $method);
-                $req->method($method);
-            }
+                    $method = $q->param($param)
+                        and $q->remove($param);
+                }
+
+                if ($method and $method =~ /^[A-Za-z]+$/) {
+                    $app->log->debug(($header // $param) . ': ' . $method);
+                    $req->method($method);
+                }
+            });
         }
     );
 }
@@ -49,7 +56,7 @@ Mojolicious::Plugin::MethodOverride - Simulate HTTP Verbs
 
 =head1 VERSION
 
-Version 0.041
+Version 0.042
 
 =head1 SYNOPSIS
 
@@ -78,6 +85,10 @@ purpose.
 Any token built of US-ASCII letters is accepted as a valid value for the
 HTTP verb.
 
+Starting with v6.03 L<Mojolicious> has a builtin parameter C<_method> to
+override the HTTP method. Unfortunately there is neither a way to specify
+another name, nor to use an HTTP header, nor to disable that feature.
+
 =head1 CONFIGURATION
 
 The default HTTP header to override the C<HTTP POST> method is
@@ -133,7 +144,7 @@ L<http://code.google.com/apis/gdata/docs/2.0/basics.html>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2012 - 2014 Bernhard Graf.
+Copyright (C) 2012 - 2015 Bernhard Graf.
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
